#!/usr/bin/env bash
#
# SCHED visibility check script v7.23
#
###############################################
#                   INPUT                     #
###############################################
#NUMBER_OF_THREADS=12
# AUTOMATICALLY_UPDATE_THE_INPUT_CATALOG = 1 -- yes, = 0 -- no
AUTOMATICALLY_UPDATE_THE_INPUT_CATALOG=1
###############################################
# don't change anything below this line unless you know what you are doing
STATIONS_DAT_FILE="stations.dat"
SOURCES_RADIOASTRON_FILE="sources.radioastron"
###############################################

# If the catalog is not in the current directory,
# maybe it is in the script directory?
#if [ ! -f "$SOURCES_RADIOASTRON_FILE" ];then
# SCRIPT_PATH=`readlink -f $0`
# SCRIPT_PATH=`dirname $SCRIPT_PATH`
# if [ -f "$SCRIPT_PATH/$SOURCES_RADIOASTRON_FILE" ];then
#  SOURCES_RADIOASTRON_FILE="$SCRIPT_PATH/$SOURCES_RADIOASTRON_FILE"
# elif [ -f "$SCRIPT_PATH/sources.radioastron" ];then
#  SOURCES_RADIOASTRON_FILE="$SCRIPT_PATH/sources.radioastron"
# fi
#fi


function set_number_of_threads {
 # Uncomment to disable multithreading
 #echo "1"
 #return 0
 #
 
 local DEFAULT_NUMBER_OF_THREADS=4

 SYSTEM_TYPE=`uname`
 if [ "$SYSTEM_TYPE" = "Linux" ];then
  NUMBER_OF_THREADS=`cat /proc/cpuinfo | grep -c 'processor'`
 else
  NUMBER_OF_THREADS=`sysctl -a | grep "hw.ncpu" | awk '{print $2}'`
 fi

 # If $OMP_NUM_THREADS is set - stick with this value
 if [ ! -z "$OMP_NUM_THREADS" ];then
  NUMBER_OF_THREADS="$OMP_NUM_THREADS"
 fi
 
 # check that NUMBER_OF_THREADS is areasonable number
 if [ -z "$NUMBER_OF_THREADS" ];then
  NUMBER_OF_THREADS=$DEFAULT_NUMBER_OF_THREADS
 fi
 TEST=`echo "$NUMBER_OF_THREADS > 0" | bc -q 2>/dev/null`
 re='^[0-9]+$'
 if ! [[ $TEST =~ $re ]] ; then
  NUMBER_OF_THREADS=$DEFAULT_NUMBER_OF_THREADS
 elif [ $TEST -eq 0 ];then
  NUMBER_OF_THREADS=$DEFAULT_NUMBER_OF_THREADS
 fi
 
 echo "$NUMBER_OF_THREADS"
 
 return 0;
}


# Find the GNU date executable: 'date' on Linux, 'gdate' on Mac 
function get_date_executable {
 command -v gdate &>/dev/null
 if [ $? -eq 0 ];then
  DATE_EXECUTABLE=gdate
 else
  DATE_EXECUTABLE=date
 fi
 echo $DATE_EXECUTABLE
}

function update_sources_radioastron_if_needed {
 if [ -z $REAL_PATH ];then
  echo "ERROR in update_sources_radioastron_if_needed: REAL_PATH is not set!" >> /dev/stderr
  return 1
 fi
 FORCE_CATALOG_UPDATE=0
 NEED_TO_UPDATE_SOURCE_CATALOG=0
 if [ -f $REAL_PATH/$SOURCES_RADIOASTRON_FILE ];then 
  #echo "Found $REAL_PATH/$SOURCES_RADIOASTRON_FILE checking how fresh is it" >> /dev/stderr
  CURRENT_DATE_UNIXSEC=`$DATE_EXECUTABLE +%s`
  SOURCES_RADIOASTRON_FILE_MODIFICATION_DATE=`stat -c %Y $REAL_PATH/$SOURCES_RADIOASTRON_FILE`
  SOURCES_RADIOASTRON_FILE_MODIFIED_SECONDS_AGO=$[$CURRENT_DATE_UNIXSEC-$SOURCES_RADIOASTRON_FILE_MODIFICATION_DATE]
  #if [ $[$CURRENT_DATE_UNIXSEC-$SOURCES_RADIOASTRON_FILE_MODIFICATION_DATE] -gt 86400 ];then
  if [ $SOURCES_RADIOASTRON_FILE_MODIFIED_SECONDS_AGO -gt 86400 ];then
   NEED_TO_UPDATE_SOURCE_CATALOG=1
  else
   #echo "The catalog file $REAL_PATH/$SOURCES_RADIOASTRON_FILE seems fresh enough: modified $SOURCES_RADIOASTRON_FILE_MODIFIED_SECONDS_AGO seconds ago" >> /dev/stderr
   NEED_TO_UPDATE_SOURCE_CATALOG=0
  fi
 else
  NEED_TO_UPDATE_SOURCE_CATALOG=1
  echo "Cannot find the source catalog $REAL_PATH/$SOURCES_RADIOASTRON_FILE -- will need to update it!" >> /dev/stderr
 fi
 if [ -z $1 ];then
  if [ "$1" = "force" ];then
   echo "WARNING: forcing the catalog update" >> /dev/stderr
   FORCE_CATALOG_UPDATE=1
   NEED_TO_UPDATE_SOURCE_CATALOG=1
  fi
 fi
 if [ $NEED_TO_UPDATE_SOURCE_CATALOG -eq 1 ];then
  RIC_DIR_URL='ftp://radik:Zap069_qq@webinet.asc.rssi.ru/radioastron/input_catalog/'
  LATEST_CATALOG_FILE=`curl --silent ftp://radik:Zap069_qq@webinet.asc.rssi.ru/radioastron/input_catalog/ | grep Radioastron_Input_Catalog_v | grep '.txt' | awk '{print $9}' | sort | tail -n1`
  if [ -s $REAL_PATH/$LATEST_CATALOG_FILE ] && [ $FORCE_CATALOG_UPDATE -eq 0 ] ;then
   echo "Found locally the catalog file $LATEST_CATALOG_FILE most likely we shouldn't download it again" >> /dev/stderr
  else
   curl --silent "$RIC_DIR_URL/$LATEST_CATALOG_FILE" > $REAL_PATH/$LATEST_CATALOG_FILE 
   if [ $? -ne 0 ];then
    echo "ERROR downloading the catalog $RIC_DIR_URL/$LATEST_CATALOG_FILE"
    return 1
   else
    echo "Successfully downloaded the catalog from $RIC_DIR_URL/$LATEST_CATALOG_FILE" >> /dev/stderr
   fi
  fi
  ############# Unicode check #############
  file "$REAL_PATH/$LATEST_CATALOG_FILE" | grep Unicode >> /dev/stderr
  if [ $? -eq 0 ];then
   echo "ERROR: non-ASCII characters in RadioAstron input catalog file $REAL_PATH/$LATEST_CATALOG_FILE" >> /dev/stderr
   echo "####### Trying to print the problematic characters #######" >> /dev/stderr
   # The following trick will not work without setting the language
   export LANG=C
   cat "$REAL_PATH/$LATEST_CATALOG_FILE" | dos2unix | dos2unix | grep --color=always '[^ -~]\+'
   echo "##########################################################" >> /dev/stderr
   return 1
  fi
  #########################################
  # We got the $REAL_PATH/$LATEST_CATALOG_FILE file one way or the other
  # now let's give it the predictable name
  cp -v $REAL_PATH/$LATEST_CATALOG_FILE $REAL_PATH/`basename $SOURCES_RADIOASTRON_FILE` >> /dev/stderr 
  if [ $? -eq 0 ];then
   echo "Successfully updated the catalog $LATEST_CATALOG_FILE" $SOURCES_RADIOASTRON_FILE >> /dev/stderr
   return 0
  else
   echo "ERROR updating the catalog $LATEST_CATALOG_FILE" $SOURCES_RADIOASTRON_FILE >> /dev/stderr
   return 1
  fi
 fi
 return 0;
}

##
##
function check_if_the_source_is_known {
 # Doesn't work if we are checking a schedule file
 #if [ -z "$1" ];then
 # echo "ERROR in check_if_the_source_is_known: target source name is not given" >> /dev/stderr
 # exit 1
 #fi
 if [ -z $REAL_PATH ];then
  echo "ERROR in check_if_the_source_is_known: REAL_PATH is not set!" >> /dev/stderr
  return 1
 fi
 SOURCE_FOUND=0
 # search in SOURCES_RADIOASTRON_FILE
 if [ -s "$REAL_PATH"/"$SOURCES_RADIOASTRON_FILE" ];then                                      
  grep -v \! "$REAL_PATH"/"$SOURCES_RADIOASTRON_FILE" | grep "$SOURCE" &>/dev/null
  if [ $? -eq 0 ];then
   SOURCE_FOUND=1
  else
   # Try to see if this source is there but commented-out
   cat "$REAL_PATH"/"$SOURCES_RADIOASTRON_FILE" | grep "$SOURCE" &>/dev/null
   if [ $? -eq 0 ];then
    #SOURCE_FOUND=1
    echo "WARNING: the source $SOURCE catalog is commented-out in $REAL_PATH/$SOURCES_RADIOASTRON_FILE" >> /dev/stderr
   fi
  fi
 else
  echo "WARNING: the source catalog is not found in $REAL_PATH/$SOURCES_RADIOASTRON_FILE" >> /dev/stderr
 fi
 # search in the default SCHED catalog
 if [ -s $SCHED/catalogs/sources.vlba ];then                                      
  grep -v \! "$SCHED/catalogs/sources.vlba" | grep "$SOURCE" &>/dev/null
  if [ $? -eq 0 ];then
   SOURCE_FOUND=1
  fi
 else
  echo "WARNING: the default SCHED catalog is not found in $SCHED/catalogs/sources.vlba" >> /dev/stderr
 fi
 # search the body of htis script
 cat $0 | grep -v \! | grep -v \# | grep "$SOURCE" &>/dev/null
 if [ $? -eq 0 ];then
  SOURCE_FOUND=1
 fi
 
 if [ $SOURCE_FOUND -eq 0 ];then
  echo "ERROR: cannot find the target source $SOURCE in $SOURCES_RADIOASTRON_FILE $SCHED/catalogs/sources.vlba $0"
  exit 1
 fi
 
}


## This function will set the random session key in attempt to avoid 
## file collisions if other instances of the script are running at the same time.
function set_session_key {
 if [ -r /dev/urandom ];then
  local RANDOMFILE=/dev/urandom
 elif [ -r /dev/random ];then  
  local RANDOMFILE=/dev/random
 else
  echo "ERROR: cannot find /dev/random" >> /dev/stderr
  local RANDOMFILE=""
 fi
 if [ "$RANDOMFILE" != "" ];then
  local SESSION_KEY="$$"_`tr -cd a-zA-Z0-9 < $RANDOMFILE | head -c 12`
 else
  local SESSION_KEY="$$"
 fi
 echo "$SESSION_KEY"
}


############# INITIAL VARIABLES #############
# Set session key
export SESSION_KEY=$(set_session_key) # or SESSION_KEY=`set_session_key`        
#
INPUT_OK=0
DETAILED_VISIBILITY_PROFILE=/tmp/autosched_detailed_visibility_profile_$SESSION_KEY.txt
REAL_PATH=`readlink -f $0`
REAL_PATH=`dirname $REAL_PATH`

#export RESULTS_DIR=/tmp/autosched_"$$"_`tr -cd a-zA-Z0-9 < /dev/urandom | head -c 12 ; echo ""`_results
export RESULTS_DIR=/tmp/autosched_"$SESSION_KEY"_results
if [ ! -d $RESULTS_DIR ];then
 mkdir $RESULTS_DIR
else
 export RESULTS_DIR=/tmp/autosched_"$SESSION_KEY"_"$SESSION_KEY"_results
 if [ ! -d $RESULTS_DIR ];then
  mkdir $RESULTS_DIR
 else
  echo "WARNING! WARNING! WARNING! The results directory $RESULTS_DIR already exists! This is not supposed to happen!" >> /dev/stderr
 fi
fi
#############################################
# Find the GNU date executable: 'date' on Linux, 'gdate' on Mac 
export DATE_EXECUTABLE=$(get_date_executable)
# Update the target source list
if [ $AUTOMATICALLY_UPDATE_THE_INPUT_CATALOG -eq 1 ];then
# echo "### Updating the RadioAstron Input Catalog ###" >> /dev/stderr
 update_sources_radioastron_if_needed
else
 echo "### NOT updating the RadioAstron Input Catalog ###
 You may set AUTOMATICALLY_UPDATE_THE_INPUT_CATALOG to 1 to enable automatic updates." >> /dev/stderr
fi
#############################################

function_to_fork() {
############ Change to a temporary directory ############
FORKED_SESSION_KEY=$(set_session_key)
#WORK_DIR=/tmp/autosched_"$$"_`tr -cd a-zA-Z0-9 < /dev/urandom | head -c 12 ; echo ""`
WORK_DIR=/tmp/autosched_"$FORKED_SESSION_KEY"
if [ -d $WORK_DIR ];then
 echo "ERROR: the temporary directory $WORK_DIR already exists ! Something is seriously messed up here!"
 exit 1
fi
mkdir $WORK_DIR
cd $WORK_DIR   
if [ $? -ne 0 ];then
 echo "ERROR: cannot change to the temporary directory $WORK_DIR ! Something is seriously messed up here!"
 exit 1
fi

echo "! SCHED visibility test
! ================================================================ 
! Control Information 
! ================================================================ 
overwrite ! Prevents need to clean out old files on restart. 
autotape = -1 ! Automatic tape allocation on 2 drive stations. 
minpause = 0 ! Minimum playback disk stop duration 
prestart = 0 ! Start disks this long before scan start 
!sumitem = early, dwell, tpstart, el1 ! Control items in summary file. 
sumitem = early, dwell, tpstart, ela ! Control items in summary file.
 
! ================================================================ 
! Cover information (PI, experiment, etc.) 
! ================================================================ 
version = 1.0 
expt = 'RadioAstron AGN ESP fringe survey' 
expcode = 're00xx' 
piname = 'Yuri Kovalev' 
address1 = 'ASC Lebedev' 
address2 = 'Profsoyuznaya 84/32' 
address3 = '117997 Moscow, Russia' 
email = 'kirx@scan.sai.msu.ru' 
phone = '+7-495-3332167' 
obsphone = '+7-915-1546281, +7-903-6614865' 
fax = '+7-495-3332378' 
obsmode = 'L/C-band, dual-pol' 
obstype = 'VLBA' 
note1 = 'L/C-band, Radioastron-compatible frequency setup' 
note2 = 'P-CAL is ON' 
note3 = '' 
note4 = '' 
 
! ================================================================ 
! Correlator section 
! ================================================================ 
correl = 'ASC' 
coravg = 1 
corchan = 32 
cornant = 6 
corpol = 'on' 
corwtfn = 'uniform' 
corsrcs = 'schedule' 
cortape = 'ftp' 
corship1 = '' 
corship2 = '' 
corship3 = '' 
cornote1 = '' 
 
! ================================================================ 
! Catalogs. These are the standard ones and are the defalts. 
! ================================================================ " > re00xx.key
if [ -s $SCHED/catalogs/sources.vlba ];then
 echo "srcfile = '$SCHED/catalogs/sources.vlba'" >> re00xx.key
fi
echo "!srcfile2 = '$REAL_PATH/$SOURCES_RADIOASTRON_FILE' 
srcfile2 = './sources.radioastron' 
 
!stafile = '$SCHED/catalogs/stations.dat' 
stafile = './stations.dat' 
!stafile = '$REAL_PATH/$STATIONS_DAT_FILE' 
 
locfile = '$SCHED/catalogs/locations.dat' 
!locfile = './locations.dat' 
 
freqfile = '$SCHED/catalogs/freq.dat' 
!freqfile = './freq.dat' 
 
! ================================================================ 
! Frequency setups 
! ================================================================ 

! ---*** 6cm frequency setup ***--- 
SETINI=ra6cm2.set /
  BAND='6CM'
  NCHAN=4
  BITS=2
  BBFILTER=16.0
  NETSIDE = L,L,U,U
! **** The RadioAstron formatter mode is $RADIOASTRON_FORMATTER_MODE - this determines the sky frequency ****
  FREQREF = $FREQREF_C
  POL=dual
  PCAL  = '$PCAL_MHZ'
!  NOISE = '$NOISE_DIODE'
  BARREL=ROLL_OFF
  /
 !--------------------------------------------------------------------------
 ! LBA stations setup should _always_ go first, otherwise it will be corrupted by other station's settings
  station   = MOPRA, ATCA, HOBART, PARKES, CEDUNA
   bbc      = 1, 1, 2, 2
   pol      = rcp, rcp, lcp, lcp
   netside  = L, U, L, U
   /
 !--------------------------------------------------------------------------
   firstlo=4500,4500,4500,4500
   ifchan  =  A,   B,   A,   B
   bbc     =  1,   9,   1,   9
   netside = U, U, L, L
   POL= RCP, LCP, RCP, LCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B  station  =  SVETLOE, ZELENCHK, BADARY /
 !--------------------------------------------------------------------------
   firstlo= 4240,4240,4240,4240
   ifchan  =   A,   C,   A,   C
   bbc     =   1,   5,   1,   5
   netside = U, U, L, L
   POL= RCP, LCP, RCP, LCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   station  =  EFLSBERG /
!--------------------------------------------------------------------------
   firstlo= 22500,22500,22500,22500
   ifchan  =   A,  B,  A,  B
   bbc    =    1,   5,   1,   5
   netside = U, U, L, L
   POL= RCP, LCP, RCP, LCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   station  =  SHAO-65 /                                                   
 !--------------------------------------------------------------------------
   firstlo= 4700,4700,4700,4700
   ifchan =  '2N','1N','2N','1N'
   bbc    =    2,   1,   2,   1
   netside = U, U, L, L
   POL= LCP, RCP, LCP, RCP     ! Reverse polarization order in WSTRBORK!
   station= WSTRBORK /
!--------------------------------------------------------------------------
   firstlo= 4158,4158,4158,4158
   ifchan  =   A,   C,   A,   C
   bbc    =    1,   2,   1,   2
   netside = U, U, L, L
   POL= RCP, LCP, RCP, LCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   station  =  YEBES40M /
!--------------------------------------------------------------------------
   firstlo= 4200,4200,4200,4200                                                                                              
   ifchan  =  A1,  B1,  A1,  B1                                                                                              
   bbc     =   1,   5,   1,   5                                                                                              
  format = MARK5B   station  =  IRBENE /                                                                                     
!
!-------------------------------------------------------------------------- 
!
   firstlo= 4200,4200,4200,4200
   ifchan =    A,   B,   A,   B
   bbc    =    1,   5,   1,   5
  format = MARK5B   station  =  KUNMING / 
!
!--------------------------------------------------------------------------
! Warning! The frequency setup for Ev is a fake! It is up to Ev staff to manually set correct parameters.
   firstlo=4500,4500,4500,4500
   ifchan  =  A,  A,  B,  B
   bbc     =  1,  1,  2,  2
   netside = U, L, U, L
   POL= LCP, LCP, RCP, RCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   ! Actually, EVPATORI records in the RDR format that is converted to MARK5B for use with DiFX.
  station  =  EVPATORI /
!--------------------------------------------------------------------------
! Warning! The frequency setup for Kl is a fake! It is up to Kl staff to manually set correct parameters.
   firstlo=4500,4500,4500,4500
   ifchan  =  A,  A,  B,  B
   bbc     =  1,  1,  2,  2
   netside = U, L, U, L
   POL= LCP, LCP, RCP, RCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   ! Actually, EVPATORI records in the RDR format that is converted to MARK5B for use with DiFX.
  station  =  KALYAZIN /
!--------------------------------------------------------------------------
! Warning! The frequency setup for Ud is a fake! It is up to Ud staff to manually set correct parameters.
   firstlo= 4240,4240,4240,4240
   ifchan  =   A,   A,   C,   C
   bbc     =   1,   1,   5,   5
   netside = U, L, U, L
   POL= LCP, LCP, RCP, RCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   station  =  USUDARA /
 !--------------------------------------------------------------------------
!
ENDSET
/

! ---*** 1.35cm frequency setup ***---
SETINI = ra1cm2.set /
  BAND='1CM'
  NCHAN=4
  BITS=2
  BBFILTER=16.0
! **** The RadioAstron formatter mode is $RADIOASTRON_FORMATTER_MODE - this determines the sky frequency ****
  FREQREF = $FREQREF_K
  FREQOFF = 0.00, 0.00, 0.00, 0.00
!  netside = L, L, U, U
$NETSIDE_LINE
  POL=dual
  PCAL  = '$PCAL_MHZ'
!  NOISE = '$NOISE_DIODE'
  BARREL=ROLL_OFF
/
 !--------------------------------------------------------------------------
 ! LBA stations setup should _always_ go first, otherwise it will be corrupted by other station's settings
  station   = MOPRA, ATCA, HOBART, PARKES, CEDUNA 
   bbc      = 1, 1, 2, 2
   pol      = rcp, rcp, lcp, lcp
   netside  = L, U, L, U
   /
 !--------------------------------------------------------------------------
 ! The TORUN K-band frequency setup is experimental! Needs to be checked!
   firstlo=21500,21500,21500,21500
   ifchan  =  C,  A,  C,  A
   bbc     =  1,  2,  1,  2
   NETSIDE = L, L, U, U
   POL= RCP, LCP, RCP, LCP
  format = MKIV   station  =  TORUN /
!--------------------------------------------------------------------------
   firstlo= 21920,21920,21920,21920
   ifchan  =    A,    B,    A,    B
   bbc     =    1,    9,    1,    9
   netside = U, U, L, L
   POL= RCP, LCP, RCP, LCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   station  =  SVETLOE, ZELENCHK, BADARY /
!--------------------------------------------------------------------------
   firstlo=21500,21500,21500,21500
   ifchan  =   A,   C,   A,   C
   bbc     =   1,   5,   1,   5
   netside = U, U, L, L
   POL= RCP, LCP, RCP, LCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   station  =  EFLSBERG /
!--------------------------------------------------------------------------
   firstlo= 21550,21550,21550,21550
   ifchan  =   A,   C,   A,   C
   bbc    =    1,   2,   1,   2
   netside = U, U, L, L
   POL= RCP, LCP, RCP, LCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   station  =  YEBES40M /
!--------------------------------------------------------------------------
! Warning! The frequency setup for Ev is a fake! It is up to Ev staff to manually set correct parameters.
   firstlo=21500,21500,21500,21500
   ifchan  =  A,  A,  B,  B
   bbc     =  1,  1,  2,  2
   netside = U, L, U, L
   POL= LCP, LCP, RCP, RCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   ! Actually, EVPATORI records in the RDR format that is converted to MARK5B for use with DiFX.
  station  =  EVPATORI /
 !--------------------------------------------------------------------------
! Warning! The frequency setup for Kl is a fake! It is up to Kl staff to manually set correct parameters.
   firstlo=21500,21500,21500,21500
   ifchan  =  A,  A,  B,  B
   bbc     =  1,  1,  2,  2
   netside = U, L, U, L
   POL= LCP, LCP, RCP, RCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   ! Actually, KALYAZIN records in the RDR format that is converted to MARK5B for use with DiFX.
  station  =  KALYAZIN /
 !--------------------------------------------------------------------------
 !--------------------------------------------------------------------------
! Warning! The frequency setup for the KVN is a fake! 
   firstlo=21500,21500,21500,21500
   ifchan  =  A,  A,  B,  B
   bbc     =  1,  1,  2,  2
   netside = U, L, U, L
   POL= LCP, LCP, RCP, RCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   ! Actually, KALYAZIN records in the RDR format that is converted to MARK5B for use with DiFX.
  station  =  KVNYS, KVNUS, KVNTN /
 !--------------------------------------------------------------------------
ENDSET
/

! ---*** 18cm setup ***---
SETINI=ra18cm2.set /
  BAND='18CM'
  NCHAN=4
  BITS=2
  BBFILTER=16.0
  NETSIDE = L,L,U,U
! **** The RadioAstron formatter mode is $RADIOASTRON_FORMATTER_MODE - this determines the sky frequency ****
  FREQREF = $FREQREF_L
  POL=dual
  $AUTOLEVEL
  PCAL     = '$PCAL_MHZ'
  NOISE = '$NOISE_DIODE'
  BARREL=ROLL_OFF
  /
 !--------------------------------------------------------------------------
 ! LBA stations setup should _always_ go first, otherwise it will be corrupted by other station's settings
  station   = MOPRA, ATCA, HOBART, PARKES, CEDUNA, WW_LBA, ASKAP 
   ! WARNING! THIS IS A FAKE
   firstlo=1260,1260,1260,1260
   ! WARNING! THIS IS A FAKE
   ifchan  =  A,   A,   B,   B
   ! The following lines are Ok
   bbc      = 1, 1, 2, 2
   pol      = rcp, rcp, lcp, lcp
   netside  = L, U, L, U
   /
!--------------------------------------------------------------------------
   firstlo=1260,1260,1260,1260
   ifchan  =  A,   B,   A,   B
   bbc     =  1,   9,   1,   9
   netside = U, U, L, L
   POL= RCP, LCP, RCP, LCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   station  =  SVETLOE, ZELENCHK, BADARY /
!--------------------------------------------------------------------------
   firstlo=1490,1490,1490,1490
   ifchan=    B,   D,   B,   D
   bbc=       1,   5,   1,   5
   netside = U, U, L, L
   POL= RCP, LCP, RCP, LCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   station  =  EFLSBERG /
!--------------------------------------------------------------------------
   firstlo=1510,1510,1510,1510
   ifchan= '2N', '1N', '2N', '1N'
   bbc    =    2,   1,   2,   1
   netside = U, U, L, L
   POL= LCP, RCP, LCP, RCP     ! Reverse polarization order in WSTRBORK!
   station= WSTRBORK /
!
   firstlo= 2324,2324,2324,2324
   ifchan =   A2,  B2,  A2,  B2
   bbc =       1,   5,   1,   5
  format = MARK5B   station  =  SARDINIA /
!
!--------------------------------------------------------------------------
! Warning! The frequency setup for Ev is a fake! It is up to Ev staff to manually set correct parameters.
   firstlo=1490,1490,1490,1490
   ifchan  =  A,  A,  B,  B
   bbc     =  1,  1,  2,  2
   netside = U, L, U, L
   POL= LCP, LCP, RCP, RCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   ! Actually, EVPATORI records in the RDR format that is converted to MARK5B for use with DiFX.
  station  =  EVPATORI /
!--------------------------------------------------------------------------
! Warning! The frequency setup for Kl is a fake! It is up to Kl staff to manually set correct parameters.
   firstlo=1490,1490,1490,1490
   ifchan  =  A,  A,  B,  B
   bbc     =  1,  1,  2,  2
   netside = U, L, U, L
   POL= LCP, LCP, RCP, RCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   ! Actually, KALYAZIN records in the RDR format that is converted to MARK5B for use with DiFX.
  station  =  KALYAZIN /
!--------------------------------------------------------------------------
! Warning! The frequency setup for Ud is a fake! It is up to Ud staff to manually set correct parameters.
   firstlo=1490,1490,1490,1490
   ifchan=    B,   B,   D,   D
   bbc=       1,   1,   5,   5
   netside = U, L, U, L
   POL= LCP, LCP, RCP, RCP     ! We specify it here explicitly, so it will not be corrupted
                               ! if a previous telescope had a different polarization setup.
  format = MARK5B   station  =  USUDARA /
 !--------------------------------------------------------------------------
ENDSET
/

! ================================================================
!       In line source catalog.
! ================================================================
srccat /
! You may add your own sources here
!source='1228+126', 'J1230+1223', '3C274', 'M87', 'VIR-A' RA=12:30:49.423382 DEC=+12:23:28.04365 equinox='J2000' rem='AGN, rfc_2013d Petrov, 2013, unpublished 55201 observations, RA-AO3-04, RA-AO3-05, RA-AO2-12' /
!source='G25.825-0.178', 'G25.825'                   RA=18:39:03.604   DEC=-06:24:11.174   equinox='J2000' rem='RA-AO5 Ivaltts' /
!
endcat /

! ================================================================ 
! The schedule 
! ================================================================ 
" >> re00xx.key

echo "year=$YEAR month=$MONTH day=$DAY start=$STARTTIME
stations=$telescope
setup=$MODE 
record 
 
source = '$SOURCE' prescan = 0 gap = 0 dur = $LENGHT_OF_THE_FIRST_SCAN_SECOND / 
" >> re00xx.key

if [ $NUMBER_OF_SCANS_MINUS_TWO -ge 1 ];then
echo "group 1 rep $NUMBER_OF_SCANS_MINUS_TWO 
 source = '$SOURCE' prescan = 0 gap = 30 dur = 570 /
" >> re00xx.key
fi 

if [ $EXPERIMENT_DURATION_MIN -ge 20 ];then
 echo "source = '$SOURCE' prescan = 0 gap = 30 dur = $LENGHT_OF_THE_LAST_SCAN_SECONDS / 
" >> re00xx.key
fi

# Make a symbolic link to stations.dat since the string $REAL_PATH/$STATIONS_DAT_FILE may be too long for SCHED to handle
ln -s "$REAL_PATH/$STATIONS_DAT_FILE" stations.dat

# Make a symbolic link to sources.radioastron since the string $REAL_PATH/$SOURCES_RADIOASTRON_FILE may be too long for SCHED to handle
ln -s "$REAL_PATH/$SOURCES_RADIOASTRON_FILE" sources.radioastron

 ##################
 # Uncomment this if you want many .key files in your /tmp directory
 # !!!!!!!!!!!!!!! #
 #echo "$WORK_DIR" >> /dev/stderr
 #exit 1
 # DEBUG DEBUG DEBUG


# Run SCHED
$SCHEDEXECUTABLE < re00xx.key | grep Finished &>/dev/null

 if [ $? -ne 0 ];then
  cd - &> /dev/null
  rm -rf $WORK_DIR &
  exit
 fi

# if [ $? -eq 0 ];then
  # If source is not visible during part of the observation
  grep "$SOURCE" re00xx.sum |grep "-" | awk '{print $6}' | grep --quiet "D" && rm -rf $WORK_DIR && exit
  grep "$SOURCE" re00xx.sum |grep "-" | awk '{print $6}' | grep --quiet "H" && rm -rf $WORK_DIR && exit
  grep "$SOURCE" re00xx.sum |grep "-" | awk '{print $6}' | grep --quiet "S" && rm -rf $WORK_DIR && exit
  grep "$SOURCE" re00xx.sum |grep "-" | awk '{print $6}' | grep --quiet "R" && rm -rf $WORK_DIR && exit
  #
  ARECIBO_SLEW_WARNING=""
  if [ "$telescope" = "ARECIBO" ];then
   grep "$SOURCE" re00xx.sum |grep "-" | awk '{print $6}' | grep --quiet "W"
   if [ $? -eq 0 ];then
    ARECIBO_SLEW_WARNING="[long_slew]"
   fi 
  else
   # Drop any telescope other than Arecibo that has the long slew warning
   grep "$SOURCE" re00xx.sum |grep "-" | awk '{print $6}' | grep --quiet "W" && rm -rf $WORK_DIR && exit
  fi
  #
  
  #TARGET_SOURCE_ELEVATION=`grep -A5000 'Bottom item is: Start elevation.' re00xx.sum | grep -A1 "$SOURCE" |grep " - " | grep -v "$SOURCE" | awk '{print $5}' | sort -n | head -n1`
  TARGET_SOURCE_ELEVATION=`grep -A5000 'Bottom item is: Average elevation.' re00xx.sum | grep -A1 "$SOURCE" |grep " - " | grep -v "$SOURCE" | awk '{print $5}' | sort -n | head -n1`

  # Get observing time
  observing_time=`grep $telescope re00xx.sum |tail -n1 | awk '{print $3" "$5}'`
  observing_dates=`grep $telescope re00xx.sum |tail -n1 | awk '{print $2" "$4}'`
  observing_date_start=`echo "$observing_dates" | awk '{print $1}'`
  observing_date_stop=`echo "$observing_dates" | awk '{print $2}'`
  # Special case of January 31
  if [ $MONTH -eq 12 ];then
   if [ $DAY -eq 31 ];then
    if [ "$observing_date_start" != "$observing_date_stop" ];then
     observing_date_stop=`echo "$observing_date_start+1"|bc -q`
    fi
   fi
  fi
  observing_date_difference=`echo "$observing_date_stop-$observing_date_start"|bc -q`
  observing_time=`echo ${observing_time//:/ }| awk '{print $1":"$2":"$3"-"$4":"$5":"$6}'`
  observing_time_start="$observing_time"
  observing_time_start=`echo ${observing_time_start//-/ }| awk '{print $1}'`
  observing_time_stop="$observing_time"
  observing_time_stop=`echo ${observing_time_stop//-/ }| awk '{print $2}'`
  observing_time_start_unixsec=`$DATE_EXECUTABLE --utc -d "$YEAR-$MONTH-$DAY  $observing_time_start" +%s`
  observing_time_stop_unixsec=`$DATE_EXECUTABLE --utc -d "$YEAR-$MONTH-$DAY  $observing_time_stop" +%s`
  observing_time_length_sec=`echo "$observing_time_stop_unixsec-$observing_time_start_unixsec+$observing_date_difference*86400"|bc -q`
  
  # If observing time lenght < intended duration of the experiment (default: 3599 sec)
  if [ $observing_time_length_sec -lt $intended_experiment_duration_seconds ];then
   cd - &> /dev/null
   rm -rf $WORK_DIR &
   exit
  fi
  
    
##  echo " $telescope" > $RESULTS_DIR/`tr -cd a-zA-Z0-9 < /dev/urandom | head -c 12 ; echo ""`$$
#  echo " $telescope" > $RESULTS_DIR/$FORKED_SESSION_KEY
  echo " $telescope($ARECIBO_SLEW_WARNING$TARGET_SOURCE_ELEVATION)" > $RESULTS_DIR/$FORKED_SESSION_KEY
# fi 


 # You may use re00xx.key as a template
 #cp $WORK_DIR/re00xx.key .
 #echo "$WORK_DIR" >> /dev/stderr
 rm -rf $WORK_DIR &


}
#### END OF FUNCTION TO FORK


################# MAIN SCRIPT ##########################

# Check if $SCHED variable is set
if [ -z $SCHED ];then
 # No SCHED variable is set, trying to guess
 if [ -d $HOME/sched ];then
  # Trying to find SCHED in the most obvious location
  export SCHED="$HOME/sched"
 else
  echo " ERROR: cannot find SCHED variable in the environment! This variable should point to SCHED installation directory. Please install SCHED." >> /dev/stderr
  exit 1
 fi
fi

# Find the SCHED executable
SCHEDEXECUTABLE=`command -v sched 2>/dev/null`
if [ -z $SCHEDEXECUTABLE ];then
 if [ -x $SCHED/bin/sched ];then
  SCHEDEXECUTABLE=$SCHED/bin/sched
 else
  echo " ERROR: cannot find sched executable! Do you have SCHED installed?" >> /dev/stderr
  exit 1
 fi
fi
# export it just in case... we'll use it from inside a forked function
export SCHEDEXECUTABLE

if [ ! -z $5 ];then
 YEAR=$1
 MONTH=$2
 DAY=$3
 STARTTIME=$4
 SOURCE=$5
 EXPERIMENT_DURATION_MIN=60
 INPUT_OK=1
fi

if [ ! -z $4 ];then
 if [ $INPUT_OK -eq 0 ];then
  YEAR=`echo $1 | awk '{print $3}' FS='.'`
  MONTH=`echo $1 | awk '{print $2}' FS='.'`
  DAY=`echo $1 | awk '{print $1}' FS='.'`
  STARTTIME=$2
  SOURCE=$3
  EXPERIMENT_DURATION_MIN=$4
  INPUT_OK=1
 fi
fi

if [ ! -z $3 ];then
 if [ $INPUT_OK -eq 0 ];then
  YEAR=`echo $1 | awk '{print $3}' FS='.'`
  MONTH=`echo $1 | awk '{print $2}' FS='.'`
  DAY=`echo $1 | awk '{print $1}' FS='.'`
  STARTTIME=$2
  SOURCE=$3
  EXPERIMENT_DURATION_MIN=60
  INPUT_OK=1
 fi
fi

# Check if the source is known, exit if not
check_if_the_source_is_known "$SOURCE"

# This is replaced with the function call below, right?!
## Find the GNU date executable: 'date' on Linux, 'gdate' on Mac 
#command -v gdate &>/dev/null
#if [ $? -eq 0 ];then
# DATE_EXECUTABLE=gdate
#else
# DATE_EXECUTABLE=date
#fi

# Find the GNU date executable: 'date' on Linux, 'gdate' on Mac 
DATE_EXECUTABLE=$(get_date_executable)

# Set the neumber of parallel threads
NUMBER_OF_THREADS=$(set_number_of_threads)

if [ ! -z $1 ];then
 if [ $INPUT_OK -eq 0 ];then

  # Clean-up
  if [ -f $DETAILED_VISIBILITY_PROFILE ];then
   rm -f $DETAILED_VISIBILITY_PROFILE
  fi

  # Automatic schedule parser to make good source names
  RAW_SCHEDULE=$1
  # Check if this was only a request to update SOURCES_RADIOASTRON_FILE
  if [ "$RAW_SCHEDULE" == "update" ];then
   exit 1
  fi
  # Check if the schedule file exist
  if [ ! -f $RAW_SCHEDULE ];then
   echo "ERROR: Cannot open schedule file $RAW_SCHEDULE" >> /dev/stderr
   exit 1
  fi

  #############################################################
  # Below we handle three possibilities:
  # 1. (default) the supplied file is a block schedule
  # 2. the supplied file is a "Yuri-style request" with UT dates
  # 3. the supplied file is a "NPO-style request" with UT+3h dates
  #############################################################

  # Check if this is a request, not an actual block schedule
  grep "start=" $RAW_SCHEDULE &> /dev/null
  if [ $? -eq 0 ];then
   grep '\[observation\]' $RAW_SCHEDULE &> /dev/null
   if [ $? -eq 0 ];then
    FORMAT="SRTSCHED (time = UT+3h)"
   else
    FORMAT="REQUEST (time = UT)"
   fi
   echo "$FORMAT format detected! Re-formating as block sched..." >> /dev/stderr
   REQUESTFILE=$RAW_SCHEDULE
   grep "start=" $REQUESTFILE |while read STRTLINE ;do
    
    SOURCE=`grep -B1 "$STRTLINE" $REQUESTFILE | head -n1 | awk '{print $1}'`
    # Check if the SOURCE line make sence
    echo "$SOURCE" | grep '\[SNS' &> /dev/null
    if [ $? -eq 0 ];then
     continue
    fi
    echo "$SOURCE" | grep 'var0' &> /dev/null
    if [ $? -eq 0 ];then
     continue
    fi
    echo "$SOURCE" | grep 'var1' &> /dev/null
    if [ $? -eq 0 ];then
     continue
    fi
    echo "$SOURCE" | grep 'var2' &> /dev/null
    if [ $? -eq 0 ];then
     continue
    fi
    echo "$SOURCE" | grep 'var3' &> /dev/null
    if [ $? -eq 0 ];then
     continue
    fi
    if [ $SOURCE == "HOME" ];then
     # This is a tracking station test, no need to check GRTs
     continue
    fi
    if [ $SOURCE == "FAKE_SRC" ];then
     # This is a tracking station test, no need to check GRTs
     continue
    fi
    
    echo "Observational code: raks01xz
Proposals: RA-AO1-10; Ef 24-12; Gb 13A-252, 13B-226; Wb R13B015; Ar V2834
Task: AGN Survey"
    if [ "$FORMAT" == "REQUEST (time = UT)" ];then
     # REQUEST format, dates are UT
     echo ${STRTLINE/start=/Start(UT):}
     STOP=`grep -A1 "$STRTLINE" $REQUESTFILE | tail -n1`
     echo ${STOP/stop =/Stop(UT) :}
    else
     # SRTSCHED format, dates are UT+3h
     OBSERVING_TIME=${STRTLINE//start=/}
     START_YEAR=`echo $OBSERVING_TIME | awk '{print $1}' | awk '{print $3}' FS='.'`
     START_MONTH=`echo $OBSERVING_TIME | awk '{print $1}' | awk '{print $2}' FS='.'`
     START_DAY=`echo $OBSERVING_TIME | awk '{print $1}' | awk '{print $1}' FS='.'`
     STARTTIME=`echo $OBSERVING_TIME | awk '{print $2}'`
     START_MSK_UNIXSEC=`$DATE_EXECUTABLE --utc -d "$START_YEAR-$START_MONTH-$START_DAY  $STARTTIME" +%s`
     START_UT_UNIXSEC=`echo "$START_MSK_UNIXSEC-3*3600" |bc -q`
     START_UT_STRING=`$DATE_EXECUTABLE --utc -d "@$START_UT_UNIXSEC" "+%d.%m.%Y %H:%M"`
     START_UT_STRING="Start(UT): $START_UT_STRING"
     
     STOP=`grep -A1 "$STRTLINE" $REQUESTFILE | tail -n1`
     OBSERVING_TIME=${STOP/stop =/}
     STOP_YEAR=`echo $OBSERVING_TIME | awk '{print $1}' | awk '{print $3}' FS='.'`
     STOP_MONTH=`echo $OBSERVING_TIME | awk '{print $1}' | awk '{print $2}' FS='.'`
     STOP_DAY=`echo $OBSERVING_TIME | awk '{print $1}' | awk '{print $1}' FS='.'`
     STOPTIME=`echo $OBSERVING_TIME | awk '{print $2}'`
     #echo "DEBUG #$STOP_DAY# #$STOPTIME#" >> /dev/stderr
     if [ "$STOPTIME" == "24:00:00" ];then
      STOPTIME="23:59:59"
     fi
     STOP_MSK_UNIXSEC=`$DATE_EXECUTABLE --utc -d "$STOP_YEAR-$STOP_MONTH-$STOP_DAY  $STOPTIME" +%s`
     STOP_UT_UNIXSEC=`echo "$STOP_MSK_UNIXSEC-3*3600" |bc -q`
     STOP_UT_STRING=`$DATE_EXECUTABLE --utc -d "@$STOP_UT_UNIXSEC" "+%d.%m.%Y %H:%M"`
     STOP_UT_STRING="Stop(UT) : $STOP_UT_STRING"
     
     echo "$START_UT_STRING
$STOP_UT_STRING"
    fi
    echo "Band: CL
Pcal: ON, noise diode: ON"
#    SOURCE=`grep -B1 "$STRTLINE" $REQUESTFILE | head -n1 | awk '{print $1}'`
    echo "Source: $SOURCE"
    GRT=`grep -A5 "$STRTLINE" $REQUESTFILE | grep GRT | tail -n1`
    echo ${GRT/GRT =/GRT: }
    echo "Comments: 13.1xED  PA=12.1
"
   done > pseudoblocksched.tmp
   RAW_SCHEDULE=pseudoblocksched.tmp
   echo "Done re-formating! The interpreted schedule is written to pseudoblocksched.tmp" >> /dev/stderr
  else  
   # Check if this is a GRT request, not an actual blocak schedule
   TEST=`grep -c "Start(UT):" $RAW_SCHEDULE`
   if [ $TEST -eq 0 ];then
    TEST=`grep -c " UT " $RAW_SCHEDULE`
    if [ $TEST -eq 0 ];then
     echo "This doesn't seem to be a GRT request..." >> /dev/stderr
     exit 1
    fi
    echo "Re-formating as block sched..." >> /dev/stderr
    REQUESTFILE=$RAW_SCHEDULE
    grep " UT" $REQUESTFILE |while read STRTLINE ;do
    
     SOURCE=`grep -A1 "$STRTLINE" $REQUESTFILE | tail -n1 | awk '{print $1}'`
     if [ $SOURCE == "HOME" ];then
      # This is a tracking station test, no need to check GRTs
      continue
     fi
     if [ $SOURCE == "FAKE_SRC" ];then
      # This is a tracking station test, no need to check GRTs
      continue
     fi
    
     echo "Observational code: raks01xz
Proposals: RA-AO1-10; Ef 24-12; Gb 13A-252, 13B-226; Wb R13B015; Ar V2834
Task: AGN Survey"

     # REQUEST format, dates are UT
     STARTDATE=`echo "$STRTLINE" | awk '{print $1}'`
     STARTTIME=`echo "$STRTLINE" | awk '{print $2}' | awk '{print $1}' FS='-'`
     STOPTIME=`echo "$STRTLINE" | awk '{print $2}' | awk '{print $2}' FS='-'`
     
     STARTDATE_DD=`echo $STARTDATE | awk '{print $1}' FS='.'`
     STARTDATE_MM=`echo $STARTDATE | awk '{print $2}' FS='.'`
     STARTDATE_YYYY=`echo $STARTDATE | awk '{print $3}' FS='.'`
     
     STARTDATE_UNIXSEC=`date -u -d "$STARTDATE_YYYY-$STARTDATE_MM-$STARTDATE_DD $STARTTIME:00" +%s`
     STOPDATE_UNIXSEC=`date -u -d "$STARTDATE_YYYY-$STARTDATE_MM-$STARTDATE_DD $STOPTIME:00" +%s`
     if [ $STOPDATE_UNIXSEC -lt $STARTDATE_UNIXSEC ];then
      STOPDATE_UNIXSEC=`echo "$STOPDATE_UNIXSEC+86400" | bc -q`
      STOPDATE_DD=`date -d @$STOPDATE_UNIXSEC +%d | awk '{printf "%02d", $1}'`
      STOPDATE_MM=`date -d @$STOPDATE_UNIXSEC +%m | awk '{printf "%02d", $1}'`
      STOPDATE_YYYY=`date -d @$STOPDATE_UNIXSEC +%Y`
      STOPDATE="$STOPDATE_DD.$STOPDATE_MM.$STOPDATE_YYYY"
     else
      STOPDATE="$STARTDATE"
     fi

     
     echo "Start(UT): $STARTDATE $STARTTIME:00"
     echo "Stop(UT) : $STOPDATE $STOPTIME:00"

     echo "Band: CL
Pcal: ON, noise diode: ON"
     echo "Source: $SOURCE"
     #GRT=`grep -A5 "$STRTLINE" $REQUESTFILE | grep GRT | tail -n1`
     GRT=`grep -A2 "$STRTLINE" $REQUESTFILE | tail -n1`
     echo "GRT: $GRT"
     echo "Comments: 13.1xED  PA=12.1
"
    done > pseudoblocksched.tmp
    #
    echo "########################################" >> /dev/stderr
    #cat pseudoblocksched.tmp >> /dev/stderr
    #
    RAW_SCHEDULE=pseudoblocksched.tmp
    echo "Done re-formating! The interpreted schedule is written to pseudoblocksched.tmp" >> /dev/stderr
   fi
  fi # SRT blocak sched or not

  echo "$0 is processing the schedule..." >> /dev/stderr

  rm -f processed_schedule.tmp
  cat $RAW_SCHEDULE | while read STR ;do
   FIRST_ITEM=`echo "$STR" | awk '{print $1}'`
   if [ "$FIRST_ITEM" == "Source:" ];then
    TARGET_SOURCE=${STR//"Source: "/}
    # Try to guess if this is a propper source name and its alias.
    # Here we assume that a propper source name would contain a "+" or "-" sign.
    if [[ "$TARGET_SOURCE" == *"+"* ]];then
     TARGET_SOURCE=`echo "$TARGET_SOURCE" | awk '{print $1}' FS=' '`
    fi
    if [[ "$TARGET_SOURCE" == *"-"* ]];then
     TARGET_SOURCE=`echo "$TARGET_SOURCE" | awk '{print $1}' FS=' '`
    fi
    # Remove white spaces from the source name
    TARGET_SOURCE=${TARGET_SOURCE//" "/""}
    # Make the source name uppercase
    TARGET_SOURCE=${TARGET_SOURCE^^}
    # Remove source alias if the alias is iside brackets
    TARGET_SOURCE=`echo "$TARGET_SOURCE" | awk '{print $1}' FS='('`
    # Remove coma from the source name
    TARGET_SOURCE=`echo "$TARGET_SOURCE" | awk '{print $1}' FS=','`
    STR="Source: $TARGET_SOURCE"
   fi
   # If the string does not contain the comment sign...
   echo "$STR" | grep "#" &>/dev/null
   if [ $? -ne 0 ];then
    echo "$STR" | sed 's:\r::g' >> processed_schedule.tmp
    #ls processed_schedule.tmp >> /dev/stderr
    #echo $PWD >> /dev/stderr
    #echo "$STR" | sed 's:\r::g' >> /dev/stderr
   fi
  done
  SCHEDULE_FILE=processed_schedule.tmp
  
  if [ ! -f $SCHEDULE_FILE ];then
   echo "ERROR: cannot find $SCHEDULE_FILE" >> /dev/stderr
   exit 1
  fi
  
  # By default, assume the schedule is OK
  rm -f problem_with_the_schedule.tmp
  
  # Check each experiment
  grep "Start(UT)" $SCHEDULE_FILE |grep -v Commen | while read STR ;do
   
   # $START_UT_STRING_FOR_GREP will be used to identify the experiment and parse its
   # description further in the scrip
   START_UT_STRING_FOR_GREP=$STR
   
   ### ***** GET EXPERIMENT DURATION ***** ###
   # Get start time
   # The following will make sure we dont care of '=' or ':' after "Start(UT)" and "Stop(UT) "
   STR=${STR//'='/':'}
   OBSERVING_TIME=${STR//"Start(UT): "/}
   YEAR=`echo $OBSERVING_TIME | awk '{print $1}' | awk '{print $3}' FS='.'`
   MONTH=`echo $OBSERVING_TIME | awk '{print $1}' | awk '{print $2}' FS='.'`
   DAY=`echo $OBSERVING_TIME | awk '{print $1}' | awk '{print $1}' FS='.'`
   STARTTIME=`echo $OBSERVING_TIME | awk '{print $2}'`
   observing_time_start_unixsec=`$DATE_EXECUTABLE --utc -d "$YEAR-$MONTH-$DAY  $STARTTIME" +%s`
   #echo "$DATE_EXECUTABLE --utc -d "$YEAR-$MONTH-$DAY  $STARTTIME" +%s" >> /dev/stderr
   # Get stop time
   #OBSERVING_TIME_STOP=`grep -A1 "$OBSERVING_TIME" $SCHEDULE_FILE | tail -n1`
   OBSERVING_TIME_STOP=`grep -A1 "$START_UT_STRING_FOR_GREP" $SCHEDULE_FILE | tail -n1`
   OBSERVING_TIME_STOP=${OBSERVING_TIME_STOP//"="/":"}
   OBSERVING_TIME_STOP=${OBSERVING_TIME_STOP//"Stop(UT) : "/}
   OBSERVING_TIME_STOP=${OBSERVING_TIME_STOP//"Stop (UT): "/}
   YEAR=`echo $OBSERVING_TIME_STOP | awk '{print $1}' | awk '{print $3}' FS='.'`
   MONTH=`echo $OBSERVING_TIME_STOP | awk '{print $1}' | awk '{print $2}' FS='.'`
   DAY=`echo $OBSERVING_TIME_STOP | awk '{print $1}' | awk '{print $1}' FS='.'`
   STOPTIME=`echo $OBSERVING_TIME_STOP | awk '{print $2}'`
   ## Dirty trick
   #STOPTIME=${STOPTIME//24:00/00:00}
   ##
   observing_time_stop_unixsec=`$DATE_EXECUTABLE --utc -d "$YEAR-$MONTH-$DAY  $STOPTIME" +%s`
   observing_time_length_sec=`echo "$observing_time_stop_unixsec-$observing_time_start_unixsec"|bc -q`
   if [ $observing_time_length_sec -le 0 ];then
    echo "Sorry, OBSERVING DATE PROBLEM. Experiment duration $observing_time_stop_unixsec-$observing_time_start_unixsec=$observing_time_length_sec<0
The offending experiment starts at $START_UT_STRING_FOR_GREP"
    exit 1
   fi
   if [ $observing_time_length_sec -gt 86400 ];then
    echo "WARNING!!! POTENTIAL OBSERVING DATE PROBLEM. Experiment duration $observing_time_stop_unixsec-$observing_time_start_unixsec=$observing_time_length_sec>1 day
The offending experiment starts at $START_UT_STRING_FOR_GREP"
    #exit 1
   fi
   EXPERIMENT_DURATION_MIN=`echo "$observing_time_length_sec/60"|bc -q`
   
   ### ***** GET OTHER DETAILS ***** ###
   #SOURCE=`grep -A7 "$OBSERVING_TIME" $SCHEDULE_FILE |grep "Source:" | head -n1`
   SOURCE=`grep -B3 -A7 "$START_UT_STRING_FOR_GREP" $SCHEDULE_FILE |grep "Source:" | head -n1`
   SOURCE=${SOURCE//"Source: "/}
   # Thank you Jenya...
   if [ "$SOURCE" == "" ];then
    # Fallback to Jenya's format: Source name before "Start(UT)"
    #SOURCE=`grep -B1 "$OBSERVING_TIME" $SCHEDULE_FILE | head -n1`
    SOURCE=`grep -B1 "$START_UT_STRING_FOR_GREP" $SCHEDULE_FILE | head -n1`
   fi
   if [ "$SOURCE" == "" ];then
    echo "Error! Cannot parse the input schedule to extract the source name!" >> /dev/stderr
    exit 1
   fi

   # Try to guess if this is a propper source name and its alias.
   # Here we assume that a propper source name would contain a "+" or "-" sign.
   if [[ "$SOURCE" == *"+"* ]];then
    SOURCE=`echo "$SOURCE" | awk '{print $1}' FS=' '`
   fi
   if [[ "$SOURCE" == *"-"* ]];then
    SOURCE=`echo "$SOURCE" | awk '{print $1}' FS=' '`
   fi
   # Remove white spaces from the source name
   SOURCE=${SOURCE//" "/""}    
   # Make the source name uppercase
   SOURCE=${SOURCE^^}
   # Remove source alias if the alias is iside brackets
   SOURCE=`echo "$SOURCE" | awk '{print $1}' FS='('`

   ### Skip HOME
   if [ "$SOURCE" == "HOME" ];then
    echo "Skipping source visibility check for the source $SOURCE..." >> /dev/stderr
    continue
   fi
   ### Skip FAKE_SRC
   if [ "$SOURCE" == "FAKE_SRC" ];then
    echo "Skipping source visibility check for the source $SOURCE..." >> /dev/stderr
    continue
   fi

   
   # Try to get expcode
   EXPCODE=`grep -B4 -A4 "$START_UT_STRING_FOR_GREP" $SCHEDULE_FILE | grep "Observational code:" | tail -n1 | awk '{print $3}'`
   
   # Get bands
   BAND=`grep -A4 "$START_UT_STRING_FOR_GREP" $SCHEDULE_FILE | grep "Band:" | tail -n1 | awk '{print $2}'`
   BAND_1=`expr substr "$BAND" 1 1`
   case "$BAND_1" in
    "K")
    ;;
    "C")
    ;;
    "L")
    ;;
    "P")
    ;;
    *)
    echo "ERROR: unknown BAND_1=$BAND_1" >> /dev/stderr
    exit 1
    ;;
   esac
   BAND_2=`expr substr "$BAND" 2 1`
   case "$BAND_2" in
    "K")
    ;;
    "C")
    ;;  
    "L")
    ;;
    "P")
    ;;
    *)
    echo "ERROR: unknown BAND_2=$BAND_2" >> /dev/stderr
    exit 1
    ;;
   esac

   #echo "$BAND $BAND_1 $BAND_2" >> /dev/stderr


   ########################
   #FAKERAD_TELESCOPES=`grep -A7 "$OBSERVING_TIME" $SCHEDULE_FILE |grep "GRT:" | head -n1`
   FAKERAD_TELESCOPES=`grep -A7 "$START_UT_STRING_FOR_GREP" $SCHEDULE_FILE |grep "GRT" | head -n1`
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"GRT="/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"GRT:"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//,/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"?"/}
   
   FAKERAD_TELESCOPES="${FAKERAD_TELESCOPES//()/($BAND_1$BAND_2)}"
   
   # INSERT
   FAKERAD_TELESCOPES_WITH_BANDS="$FAKERAD_TELESCOPES"
   # END OF INSERT
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(PL)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(LP)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(PC)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(CP)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(PK)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(KP)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(KL)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(LK)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(CL)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(LC)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(KC)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(CK)"/}
   #
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(L)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(C)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(K)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(P)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(P)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(C->K)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(K->C)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(C->L)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(L->C)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(K->L)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(L->K)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(P->L)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(L->P)"/}
   #
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(X)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(U)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(U->X)"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"(X->U)"/}
   #
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"("/" "}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//")"/" "}
   #
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"+"/}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"-"/}
   #echo "$0 $OBSERVING_TIME $SOURCE $EXPERIMENT_DURATION_MIN" >> /dev/stderr
   SCHED_TELESCOPES=`$0 $OBSERVING_TIME $SOURCE $EXPERIMENT_DURATION_MIN 2>/dev/stderr`
   echo "$OBSERVING_TIME $SOURCE  $EXPCODE  $SCHED_TELESCOPES" >> $DETAILED_VISIBILITY_PROFILE
   ###
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"Jb1"/"Jb"}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"Jb2"/"Jb"}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"Y27"/"Y"}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"Wb1"/"Wb"}
   ###
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"T6"/"Tm"}
   ###
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"On85"/"On"}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"On60"/"On"}
   ###
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"Ro70"/"Ro"}
   ###
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"Eu"/"Ev"}
   ###
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"VLBA"/"Pt Kp La Br Fd Sc Nl Ov Mk Hn"}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"LBA"/"Pa At Td Mp Ho Cd Hh Ww Ak"}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"KVN"/"Ky Ku Kt"}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"EVN"/"Ef Wb Jb Mc Nt Ys On Mh Tr Sv Zc Bd Ur Sh "}
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"ATCA"/"At"}
   ###
   FAKERAD_TELESCOPES=${FAKERAD_TELESCOPES//"Ib"/"Ir"}
   #echo FAKERAD_TELESCOPES=$FAKERAD_TELESCOPES >> /dev/stderr
   #exit # !!!!

   # See if we can check observing bands
   # CHECK_OBSERVING_BANDS = 1 - yes, 0 - no
   CHECK_OBSERVING_BANDS=1
   echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "($BAND_1" &>/dev/null
   if [ $? -ne 0 ];then
    #echo "The telescopes line ($FAKERAD_TELESCOPES_WITH_BANDS) does not seem to contain information about observing bands! Skipping the band check." >> /dev/stderr
    CHECK_OBSERVING_BANDS=0
   fi
   echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "($BAND_2" &>/dev/null
   if [ $? -ne 0 ];then
    #echo "The telescopes line ($FAKERAD_TELESCOPES_WITH_BANDS) does not seem to contain information about observing bands! Skipping the band check." >> /dev/stderr
    CHECK_OBSERVING_BANDS=0
   fi

   for i in $FAKERAD_TELESCOPES ;do
    #echo "DEBUG: FAKERAD_TELESCOPES=#$FAKERAD_TELESCOPES#"
    
    # Check if this telescope is not canceled
    echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "$i-" &>/dev/null
    if [ $? -eq 0 ];then
     continue
    fi
    
    # Try to avoid telescope names that are not actually telescope names
    echo "$i" | grep 'P:' &> /dev/null
    if [ $? -eq 0 ];then
     continue
    fi
    echo "$i" | grep 'L:' &> /dev/null
    if [ $? -eq 0 ];then
     continue
    fi
    echo "$i" | grep 'C:' &> /dev/null
    if [ $? -eq 0 ];then
     continue
    fi
    echo "$i" | grep 'K:' &> /dev/null
    if [ $? -eq 0 ];then
     continue
    fi
    #
    echo "$i" | grep 'PP' &> /dev/null
    if [ $? -eq 0 ];then
     continue
    fi
    echo "$i" | grep 'LL' &> /dev/null
    if [ $? -eq 0 ];then
     continue
    fi
    echo "$i" | grep 'KK' &> /dev/null
    if [ $? -eq 0 ];then
     continue
    fi
    #    
    if [ "$i" == "P" ];then
     continue
    fi
    if [ "$i" == "L" ];then
     continue
    fi
    if [ "$i" == "C" ];then
     continue
    fi
    if [ "$i" == "K" ];then
     continue
    fi
    #
    # Make sure telescope's name does not contain ':'
    echo "$i" | grep ':' &>/dev/null
    if [ $? -eq 0 ];then
     continue
    fi
    # Make sure telescope's name is not longer than 3 characters
    if [ ${#i} -gt 3 ];then
     echo "$OBSERVING_TIME $SOURCE   $i <-- UNRECOGNIZED TELESCOPE NAME LONGER THAN 3 CHARACTERS"
     continue
    fi
    # Special case of "Eb" = "Ef"
    if [ "$i" == "Eb" ];then
     i="Ef"
    fi
    # Special case of "Y27" = "Y"
    if [ "$i" == "Y27" ];then
     i="Y"
    fi



    # Check if the requested telescope is among those that can observe according to SCHED
    TELESCOPE_OK=0
    #echo "DEBUG: #$i# #$SCHED_TELESCOPES#" >> /dev/stderr
    for j in $SCHED_TELESCOPES ;do
     # this is to accomodate the new Telescope(Elevation) format
     j=`echo "$j" | awk '{print $1}' FS='('`
     #
     if [ "$j" == "$i" ];then
      TELESCOPE_OK=1
      break
     fi
     ### NOT NEEDED
     ### Special case
     #if [ "$j" == "Ir" ] && [ "$i" == "Ib" ];then
     # TELESCOPE_OK=1
     # break
     #fi
     #if [ "$j" == "Ib" ] && [ "$i" == "Ir" ];then
     # TELESCOPE_OK=1
     # break
     #fi
     ###
    done
    if [ $TELESCOPE_OK -eq 0 ];then
     echo "$OBSERVING_TIME $SOURCE   $i <-- VISIBILITY NOT OK"
     touch problem_with_the_schedule.tmp
     continue
    fi
    
    # See if we should skip the following observing bands check
    if [ $CHECK_OBSERVING_BANDS -eq 0 ];then
     continue
    fi

    #### Check band consistency
    # by default we are not OK
    BAND_CONSISTENCY_OK=0
    if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "$i($BAND_1" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi
    if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "$i($BAND_2" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi
    if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "$i($BAND_1->$BAND_2)" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi
    if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "$i($BAND_2->$BAND_1)" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi
    if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "$i($BAND_1$BAND_2)" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi
    if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "$i($BAND_2$BAND_1)" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi
    #
    if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "${i/Jb/Jb1}($BAND_1" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi
    if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "${i/Jb/Jb2}($BAND_2" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi
    if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "${i/Tm/T6}($BAND_1" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi
    if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "${i/Tm/T6}($BAND_2" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi    
    #
    if [ "$i" == "On" ];then
     if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "${i/On/On60}($BAND_1" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi
     if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "${i/On/On60}($BAND_2" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi
     if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "${i/On/On85}($BAND_1" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi
     if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "${i/On/On85}($BAND_2" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi
    fi
    # check also Ib
    if [ "$i" == "Ir" ];then
     if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "${i/Ir/Ib}($BAND_1" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi
     if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "${i/Ir/Ib}($BAND_2" >/dev/null ;then BAND_CONSISTENCY_OK=1 ;fi
    fi
    ##################################
    # Check if an arbitrary number in the range 1 - 100 is added to the telescope name
    # Yes, names like Wb1 and On85 are so coooool...
    for SILLY_NUMBER in `seq 1 100` ;do
     if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "${i/$i/$i$SILLY_NUMBER}($BAND_1" >/dev/null ;then BAND_CONSISTENCY_OK=1 ; break ;fi
     if echo "$FAKERAD_TELESCOPES_WITH_BANDS" | grep "${i/$i/$i$SILLY_NUMBER}($BAND_2" >/dev/null ;then BAND_CONSISTENCY_OK=1 ; break ;fi
    done
    ##################################
    #
    if [ $BAND_CONSISTENCY_OK -eq 0 ];then
     echo "$OBSERVING_TIME $SOURCE   $i <-- OBSERVING BAND PROBLEM! Expected bands: $BAND_1$BAND_2 Telescopes string: $FAKERAD_TELESCOPES_WITH_BANDS"
    fi
    ####

   done
  done
  # Report a problem if there is one
  echo "----------------------------------------------------" >> /dev/stderr
  if [ -f problem_with_the_schedule.tmp ];then
   echo "It seems there was a problem with the schedule... :(" >> /dev/stderr
  else
   echo "It seems the schedule is Ok... :)" >> /dev/stderr
  fi
  echo "A detailed visibility profile for this schedule may be found in $DETAILED_VISIBILITY_PROFILE" >> /dev/stderr
  rm -f processed_schedule.tmp
  exit
 fi
fi


# Check format of dates
YEAR=`echo $YEAR | awk '{printf "%04d",$1}'`
MONTH=`echo $MONTH | awk '{printf "%02d",$1}'`
DAY=`echo $DAY | awk '{printf "%02d",$1}'`

# Check if seconds are supplied
START_HH=`echo "$STARTTIME" | awk '{printf "%02d", $1}' FS=':'`
START_MM=`echo "$STARTTIME" | awk '{printf "%02d", $2}' FS=':'`
START_SS=`echo "$STARTTIME" | awk '{print $3}' FS=':'`
if [ ! -z $START_SS ];then
 START_SS="00"
fi
STARTTIME=`echo "$START_HH $START_MM $START_SS" | awk '{printf "%02d:%02d:%02d",$1,$2,$3}'`

# Check if the input is Ok
if [ $INPUT_OK -eq 0 ];then
 echo "Usage: $0 31.12.2012 21:20:00 0716+714 40
 # DATE TIME SOURCE EXPERIMENT_DURATION_MINUTES
 or
 $0 31.12.2012 21:20:00 0716+714
 or
 $0 2012 12 31 21:20 0716+714"  >> /dev/stderr
 exit 1
fi

######
#echo DEBUG: $0 $1 $2 $3 $4 $5 $6 $7 $8 $9 >> /dev/stderr
######

#
###############################################
# Check the input

if [ -z $YEAR ];then
 echo "Please set year!" >> /dev/stderr
 exit 1
fi
if [ -z $MONTH ];then
 echo "Please set month!" >> /dev/stderr
 exit 1
fi
if [ -z $DAY ];then
 echo "Please set day!" >> /dev/stderr
 exit 1
fi
if [ -z $STARTTIME ];then
 echo "Please set start time!" >> /dev/stderr
 exit 1
fi
if [ -z $SOURCE ];then
 echo "Please set the source name!" >> /dev/stderr
 exit 1
fi

#echo "Input parameters:  YEAR=$YEAR MONTH=$MONTH DAY=$DAY STARTTIME=$STARTTIME SOURCE=$SOURCE EXPERIMENT_DURATION_MIN=$EXPERIMENT_DURATION_MIN" >> /dev/stderr
###############################################



# Compute the intended experiment duration in seconds
intended_experiment_duration_seconds=`echo "$EXPERIMENT_DURATION_MIN*60-1" |bc -q`
# We want one second to spare if the exparement ends at 23:59:59

# Make sure we don't loose a 10 minute scan because of one minute
if [ $EXPERIMENT_DURATION_MIN -eq 59 ];then
 EXPERIMENT_DURATION_MIN=60
elif [ $EXPERIMENT_DURATION_MIN -eq 49 ];then
 EXPERIMENT_DURATION_MIN=50
elif [ $EXPERIMENT_DURATION_MIN -eq 39 ];then
 EXPERIMENT_DURATION_MIN=40
elif [ $EXPERIMENT_DURATION_MIN -eq 29 ];then
 EXPERIMENT_DURATION_MIN=20
fi

### NOW, the new trick: choose the length of the last scan
LENGHT_OF_THE_LAST_SCAN_SECONDS=`echo "600+${EXPERIMENT_DURATION_MIN: (-1)}*60" |bc -q`
LENGHT_OF_THE_FIRST_SCAN_SECOND=570
if [ $EXPERIMENT_DURATION_MIN -lt 20 ];then
 LENGHT_OF_THE_FIRST_SCAN_SECOND=`echo "$EXPERIMENT_DURATION_MIN*60" | bc -q`
fi

## Make sure the experiment duration is a multiple of 10'
#if [ ${EXPERIMENT_DURATION_MIN:(-1)} -ne 0 ];then
# echo "ERROR: the experiment duration sould be a multiple of 10 minutes:
#40 minutes - good, 45 minutes - not good." >/dev/stderr
# exit 1
#fi

# Compute the number of 10' scans
NUMBER_OF_SCANS_MINUS_TWO=`echo "$EXPERIMENT_DURATION_MIN/10-2" |bc -q`
#echo "EXPERIMENT_DURATION_MIN=$EXPERIMENT_DURATION_MIN  NUMBER_OF_SCANS_MINUS_TWO=$NUMBER_OF_SCANS_MINUS_TWO" >> /dev/stderr
if [ $NUMBER_OF_SCANS_MINUS_TWO -lt -1 ] ;then
 if [ $EXPERIMENT_DURATION_MIN -lt 1 ];then
  echo "Error: unsupported observation length! Please try observation length of 1, 2, 5, 10, 20, 40, 60 minutes..." >> /dev/stderr
  exit 1
 fi
# if [ $EXPERIMENT_DURATION_MIN -gt 10 ];then
#  echo "Error: unsupported observation length! Please try 5, 10, 20, 40, 60 minutes..." >> /dev/stderr
#  exit 1
# fi
fi


## Make SOURCE uppercase
SOURCE=${SOURCE^^}
## Special case of BL-LAC
if [ "$SOURCE" == "BL-LAC" ];then
 SOURCE="BLLAC"
fi
## Special case of SNS_LL
echo "$SOURCE" | grep "SNS_LL" &> /dev/null
if [ $? -eq 0 ];then
 # This is a laser ranging session, not a source
 exit
fi

# Check the source name length
SOURCE_NAME_LENGTH=${#SOURCE}
if [ $SOURCE_NAME_LENGTH -gt 12 ];then
 echo "ERROR: SCHED might not understand the source name longer than 12 characters if it's included from an external catalog.
The source name '$SOURCE' is $SOURCE_NAME_LENGTH characters long.
Please use a shorter name for this source." >> /dev/stderr
 exit 1
elif [ $SOURCE_NAME_LENGTH -gt 10 ];then
 echo "WARNING: the older versions of the RDR recorder control software might not understand source names longer than 10 characters.
The source name '$SOURCE' is $SOURCE_NAME_LENGTH characters long.
Please consider using a shorter name for this source." >> /dev/stderr
fi

# Check if the source is in the catalog
grep --ignore-case \'$SOURCE\' $REAL_PATH/$SOURCES_RADIOASTRON_FILE &> /dev/null
if [ $? -ne 0 ];then
 # Check if SCHED knows the source
 grep --ignore-case \'$SOURCE\' $SCHED/catalogs/sources.vlba &> /dev/null
 if [ $? -ne 0 ];then
  # Source not found in the SCHED's catalog
  # Maybe it is in the script's internal catalog?
  grep -v \! $0 | grep --ignore-case \'$SOURCE\' &> /dev/null
  if [ $? -ne 0 ];then
   # Source was not found, report the problem
   echo "ERROR: source \"$SOURCE\" not found!" >> /dev/stderr
   # Now try to suggest a solution
   GUESSED_SOURCE_NAME_FOUND=0
 
   # Now try the few obvious source name modification
   GUESSED_SOURCE_NAME="$SOURCE"_H2O
   grep -v \! $0 | grep --ignore-case \'$GUESSED_SOURCE_NAME\' &> /dev/null
   if [ $? -eq 0 ];then
    echo "Did you mean \"$GUESSED_SOURCE_NAME\"?" >> /dev/stderr
    GUESSED_SOURCE_NAME_FOUND=1
   fi
   
   GUESSED_SOURCE_NAME=${SOURCE//"_H2O"/}
   grep -v \! $0 | grep --ignore-case \'$GUESSED_SOURCE_NAME\' &> /dev/null
   if [ $? -eq 0 ];then
    echo "Did you mean \"$GUESSED_SOURCE_NAME\"?" >> /dev/stderr
    GUESSED_SOURCE_NAME_FOUND=1
   fi
   
   GUESSED_SOURCE_NAME="$SOURCE"_OH
   grep -v \! $0 | grep --ignore-case \'$GUESSED_SOURCE_NAME\' &> /dev/null
   if [ $? -eq 0 ];then
    echo "Did you mean \"$GUESSED_SOURCE_NAME\"?" >> /dev/stderr
    GUESSED_SOURCE_NAME_FOUND=1
   fi
   
   GUESSED_SOURCE_NAME=${SOURCE//"_OH"/}
   grep -v \! $0 | grep --ignore-case \'$GUESSED_SOURCE_NAME\' &> /dev/null
   if [ $? -eq 0 ];then
    echo "Did you mean \"$GUESSED_SOURCE_NAME\"?" >> /dev/stderr
    GUESSED_SOURCE_NAME_FOUND=1
   fi
   
 
   # If no good names were found, just tell the user to hack off
   if [ $GUESSED_SOURCE_NAME_FOUND -eq 0 ];then
    echo "Please add the source to the catalog inside the body of the script: $0" >> /dev/stderr
   fi
   exit 1
  fi
 fi
fi


## !!! Special trap to work with ./draft.sh
#if [ -f sched_visibility_test/stations.dat ];then
# STATIONS_DAT_FILE="sched_visibility_test/stations.dat"
#fi

# If there is no local copy of stations.dat
if [ ! -f $REAL_PATH/$STATIONS_DAT_FILE ];then
 echo "WARNING: cannot find stations.dat file! Thrying to download one..." >> /dev/stderr
 wget -O $REAL_PATH/$STATIONS_DAT_FILE "ftp://grt:K0&th%@webinet.asc.rssi.ru/schedule/grtsched/RAES/rk01aa/stations.dat" &>/dev/stderr
fi

# If there is no local copy of stations.dat and we couldn't download it
if [ ! -f $REAL_PATH/$STATIONS_DAT_FILE ];then
 echo "WARNING: no custom-made stations.dat file is found in "`dirname $0`" This means the script cannot check the funny antennas EVPATORI and USUDARA" >> /dev/stderr
 cp $SCHED/catalogs/stations.dat $REAL_PATH/$STATIONS_DAT_FILE
fi

# If still there is no file
if [ ! -f $REAL_PATH/$STATIONS_DAT_FILE ];then
 echo "ERROR! Cannot find stations.dat" >> /dev/stderr
 exit 1
fi

# Go!
telescopes=""
# Full list of telescopes
for telescope in SVETLOE ZELENCHK BADARY EVPATORI EFLSBERG GBT_VLBA WSTRBORK MEDICINA NOTO YEBES40M TORUN HART JODRELL1 SHANGHAI SHAO-65 URUMQI ONSALA85 METSAHOV USUDARA DSS63 ATCA WW_LBA MOPRA HOBART CEDUNA PARKES DSS43 VLBA_PT VLBA_KP VLBA_LA VLBA_BR VLBA_FD VLBA_SC VLBA_NL VLBA_OV VLBA_MK VLBA_HN VLA27 ARECIBO KVNYS KVNUS KVNTN KALYAZIN SARDINIA ASKAP KASHIM34 IRBENE KUNMING ;do
# Shorter list
#for telescope in SVETLOE ZELENCHK BADARY EFLSBERG GBT_VLBA WSTRBORK MEDICINA NOTO YEBES40M TORUN HART JODRELL1 SHAO-65 URUMQI ONSALA85 METSAHOV DSS63 ATCA WW_LBA MOPRA HOBART CEDUNA PARKES DSS43 VLBA_PT VLBA_KP VLBA_LA VLBA_BR VLBA_FD VLBA_SC VLBA_NL VLBA_OV VLBA_MK VLBA_HN VLA27 ARECIBO KVNYS KVNUS KVNTN SARDINIA KASHIM34 IRBENE KUNMING ;do
# Just one telescope for testing
#for telescope in ARECIBO ;do
 # just in case
 #export telescope=$telescope

 # Default settings
 FREQREF_K="22236.00"
 FREQREF_C="4836.00"
 FREQREF_L="1668.00"
 PCAL_MHZ="1MHz"
 NOISE_DIODE="ON"

 # Default mode
 MODE="ra6cm2.set"
 NETSIDE_LINE="  netside = L, L, U, U"
 # Modes for special telescopes that do not work in the default mode
 if [ "$telescope" == "DSS63" ];then
  MODE="ra1cm2.set"
 elif [ "$telescope" == "DSS43" ];then
  MODE="ra1cm2.set"
 #elif [ "$telescope" == "DSS14" ];then
 # MODE="ra1cm2.set"
 elif [ "$telescope" == "METSAHOV" ];then
  MODE="ra1cm2.set"
 elif [ "$telescope" == "VLA27" ];then
  MODE="ra1cm2.set"
  # A different netside line is needed to trick the VLA
  NETSIDE_LINE="  netside = U, U"
 elif [ "$telescope" == "KVNYS" ];then
  MODE="ra1cm2.set"
 elif [ "$telescope" == "KVNUS" ];then
  MODE="ra1cm2.set"
 elif [ "$telescope" == "KVNTN" ];then
  MODE="ra1cm2.set"
 elif [ "$telescope" == "WW_LBA" ];then
  MODE="ra18cm2.set"
 elif [ "$telescope" == "ASKAP" ];then
  MODE="ra18cm2.set"
 elif [ "$telescope" == "JODRELL1" ];then
  MODE="ra18cm2.set"
 elif [ "$telescope" == "SARDINIA" ];then
  MODE="ra18cm2.set"
 fi

############

 # Go multithread
 #echo $telescope >> /dev/stderr
 function_to_fork &
 PIDS="$PIDS $! "
 NPROC=$[$NPROC+1]
 while [ $NPROC -gt $[$NUMBER_OF_THREADS-1] ] ;do
  # check each running process
  #echo "$PIDS"
  for PID in $PIDS ;do
   ps -p $PID &> /dev/null
   if [ $? -ne 0 ];then   
    PIDS=${PIDS/" $PID "/}
    NPROC=$[$NPROC-1]
   fi
  done
  sleep 0.01
 done
 # Now we are sure no more than NUMBER_OF_THREADS procssess are runnning
 #echo "$PIDS" > /tmp/autosched_"$$"_pids.tmp
 echo "$PIDS" > /tmp/autosched_"$SESSION_KEY"_pids.tmp

done

# Wait for the remeining processes to finish
#for PID in `cat /tmp/autosched_"$$"_pids.tmp` ;do
for PID in `cat /tmp/autosched_"$SESSION_KEY"_pids.tmp` ;do
 while true ;do
  ps -p $PID &> /dev/null
  if [ $? -ne 0 ];then   
   break
  fi
  sleep 0.01
 done
done 
# In case something above fails...
wait
#rm -f /tmp/autosched_"$$"_pids.tmp &
rm -f /tmp/autosched_"$SESSION_KEY"_pids.tmp &


#echo "$telescopes"
if [ "$(ls -A $RESULTS_DIR)" ]; then
 cat $RESULTS_DIR/* | sort > $RESULTS_DIR.tmp 
else
 echo "NO_TELESCOPES" | sort > $RESULTS_DIR.tmp
fi

while read telescope ;do
  #
  telescope_and_elevation="$telescope"
  telescope=`echo "$telescope" | awk '{print $1}' FS='('`
  #
  short_telescope=""
  if [ "$telescope" == "SVETLOE" ];then
   short_telescope="Sv"
  elif [ "$telescope" == "ZELENCHK" ];then
   short_telescope="Zc"
  elif [ "$telescope" == "BADARY" ];then
   short_telescope="Bd"
  elif [ "$telescope" == "EVPATORI" ];then
   short_telescope="Ev"
  elif [ "$telescope" == "KALYAZIN" ];then
   short_telescope="Kl"
  elif [ "$telescope" == "EFLSBERG" ];then
   short_telescope="Ef"
  elif [ "$telescope" == "GBT_VLBA" ];then
   short_telescope="Gb"
  #elif [ "$telescope" == "GBT_COLD" ];then
  # short_telescope="Gb"
  elif [ "$telescope" == "WSTRBORK" ];then
   short_telescope="Wb"
  elif [ "$telescope" == "MEDICINA" ];then
   short_telescope="Mc"
  elif [ "$telescope" == "NOTO" ];then
   short_telescope="Nt"
  elif [ "$telescope" == "YEBES40M" ];then
   short_telescope="Ys"
  elif [ "$telescope" == "JODRELL1" ];then
   short_telescope="Jb"
  elif [ "$telescope" == "SHANGHAI" ];then
   short_telescope="Sh"
  elif [ "$telescope" == "SHAO-65" ];then
   short_telescope="Tm"
  elif [ "$telescope" == "URUMQI" ];then
   short_telescope="Ur"
  elif [ "$telescope" == "TORUN" ];then
   short_telescope="Tr"
  elif [ "$telescope" == "ONSALA85" ];then
   short_telescope="On"
  elif [ "$telescope" == "METSAHOV" ];then
   short_telescope="Mh"
  elif [ "$telescope" == "ARECIBO" ];then
   short_telescope="Ar"
  elif [ "$telescope" == "USUDARA" ];then
   short_telescope="Ud"
  elif [ "$telescope" == "DSS63" ];then
   short_telescope="Ro"
  elif [ "$telescope" == "ATCA" ];then
   short_telescope="At"
  elif [ "$telescope" == "CEDUNA" ];then
   short_telescope="Cd"
  elif [ "$telescope" == "DSS43" ];then
   short_telescope="Td"
  #elif [ "$telescope" == "DSS14" ];then
  # short_telescope="Go"
  elif [ "$telescope" == "ASKAP" ];then
   short_telescope="Ak"
  elif [ "$telescope" == "WW_LBA" ];then
   short_telescope="Ww"
  elif [ "$telescope" == "HART" ];then
   short_telescope="Hh"
  elif [ "$telescope" == "HOBART" ];then
   short_telescope="Ho"
  elif [ "$telescope" == "MOPRA" ];then
   short_telescope="Mp"
  elif [ "$telescope" == "PARKES" ];then
   short_telescope="Pa"
  elif [ "$telescope" == "VLBA_PT" ];then
   short_telescope="Pt"
  elif [ "$telescope" == "VLBA_KP" ];then
   short_telescope="Kp"
  elif [ "$telescope" == "VLBA_LA" ];then
   short_telescope="La"
  elif [ "$telescope" == "VLBA_BR" ];then
   short_telescope="Br"
  elif [ "$telescope" == "VLBA_FD" ];then
   short_telescope="Fd"
  elif [ "$telescope" == "VLBA_SC" ];then
   short_telescope="Sc"
  elif [ "$telescope" == "VLBA_NL" ];then
   short_telescope="Nl"
  elif [ "$telescope" == "VLBA_OV" ];then
   short_telescope="Ov"
  elif [ "$telescope" == "VLBA_MK" ];then
   short_telescope="Mk"
  elif [ "$telescope" == "VLBA_HN" ];then
   short_telescope="Hn"
  elif [ "$telescope" == "VLA27" ];then
   short_telescope="Y"
  elif [ "$telescope" == "KVNYS" ];then
   short_telescope="Ky"
  elif [ "$telescope" == "KVNUS" ];then
   short_telescope="Ku"
  elif [ "$telescope" == "KVNTN" ];then
   short_telescope="Kt"
  elif [ "$telescope" == "SARDINIA" ];then
   short_telescope="Sr"
  elif [ "$telescope" == "KASHIM34" ];then
   short_telescope="Ks"
  elif [ "$telescope" == "IRBENE" ];then
   short_telescope="Ir"
  elif [ "$telescope" == "KUNMING" ];then
   short_telescope="Km"
  else
   short_telescope="$telescope"
  fi
 #echo -n " $short_telescope" 
 echo -n " ${telescope_and_elevation/$telescope/$short_telescope}"
done < $RESULTS_DIR.tmp
echo ""

rm -rf $RESULTS_DIR $RESULTS_DIR.tmp 

